@@ -21,10 +21,7 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define PY_SSIZE_T_CLEAN
 
-// #include <libpymath/LibPyMathModules/internalFunctions.h>
-#include <Python.h>
-
-#define internalGet(i, j, r, c) ((j) * (c) + (i) * (r))
+#include <libpymath/LibPyMathModules/internalFunctions.h>
 
 // ************************************************************************************************************************* //
 // ==================================================== Error Functions ==================================================== //
